<?php
/***************************************************************************************************************************************************************************
 * 返回信息封装
 ***************************************************************************************************************************************************************************/
function returnMsg($_err_msg,$_res_code = '1')
{
    return array('res_code'=>$_res_code,'err_msg'=>$_err_msg);
}
/***************************************************************************************************************************************************************************
 * 手机验证码相关
 ***************************************************************************************************************************************************************************/
/**
* 验证手机号是否正确
* @author honfei
* @param number $mobile
*/
function isMobile($mobile) {
    if (!is_numeric($mobile)) {
        return false;
    }
    return preg_match('#^13[\d]{9}$|^14[5,7]{1}\d{8}$|^15[^4]{1}\d{8}$|^17[0,6,7,8]{1}\d{8}$|^18[\d]{9}$#', $mobile) ? true : false;
}

/***************************************************************************************************************************************************************************
 * 日志记录相关
 ***************************************************************************************************************************************************************************/
/**
 * 构造错误数据体
 * @param type $_errMsg
 * @param type $_time
 * @param type $_clientIP
 * @param type $_clientType
 */
function buildErrLogData($_errMsg,$_time,$_clientIP,$_clientType,$_errMark = '0')
{
    $data['error_msg'] = $_errMsg;
    $data['time'] = $_time;
    if(isVarEmpty($_time))
    {
        $data['time'] = time();
    }
    $data['client_ip'] = $_clientIP;
    $data['client_type'] = $_clientType;
    $data['error_mark'] = $_errMark;
    
    return $data;
}

/**
 * 判断请求的平台类型（各种浏览器的标识）
 * WX - 微信内置浏览器
 */
function getPlatFormType()
{
    $ua = $_SERVER["HTTP_USER_AGENT"];
    if(isWeixin())
    {
        return 'WX';
    }
    else if(strpos($ua,"Weibo"))	//微博
    {
        return 'WB';
    }
    else if(strpos($ua,"AliApp"))	//来往
    {
        return 'LW';
    }
    else
    {
        return 'OTHER';
    }
}
/**
 * 获取请求设备类型（0-手机/1-PC）
 */
function getDeviceType()
{
    if(isCellPhone())
    {
        return '0';
    }
    return '1';
}
function isCellPhone(){
    $useragent=isset($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : '';
    $useragent_commentsblock=preg_match('|\(.*?\)|',$useragent,$matches)>0?$matches[0]:'';
    
    $mobile_os_list=array('Google Wireless Transcoder','Windows CE','WindowsCE','Symbian','Android','armv6l','armv5','Mobile','CentOS','mowser','AvantGo','Opera Mobi','J2ME/MIDP','Smartphone','Go.Web','Palm','iPAQ');
    $mobile_token_list=array('Profile/MIDP','Configuration/CLDC-','160×160','176×220','240×240','240×320','320×240','UP.Browser','UP.Link','SymbianOS','PalmOS','PocketPC','SonyEricsson','Nokia','BlackBerry','Vodafone','BenQ','Novarra-Vision','Iris','NetFront','HTC_','Xda_','SAMSUNG-SGH','Wapaka','DoCoMo','iPhone','iPod');
     
    $res = CheckSubstrs($mobile_os_list,$useragent_commentsblock) || CheckSubstrs($mobile_token_list,$useragent);

    if ($res){
        return true;
    }else{  
        return false;
    }
}
function CheckSubstrs($substrs,$text){
    foreach($substrs as $substr){
        if(false!==strpos($text,$substr)){
            return true;
        }else{
            return false;
        }
    }
}

function https_request($url,$data = null){
    $curl = curl_init();
    curl_setopt($curl, CURLOPT_URL, $url);
    curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, FALSE);
    curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, FALSE);
    if (!empty($data)){
        curl_setopt($curl, CURLOPT_POST, 1);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
    }
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
    $output = curl_exec($curl);
    curl_close($curl);
    return $output;
}


/**
 * 获取客户端IP
 */
function getClientRealIp()
{
    if (isset($_SERVER['HTTP_CDN_SRC_IP']) && $_SERVER['HTTP_CDN_SRC_IP'] != 'unknown') {
        $realip = $_SERVER["HTTP_CDN_SRC_IP"];
    } elseif (getenv('HTTP_X_FORWARDED_FOR')) {
        $realip = getenv('HTTP_X_FORWARDED_FOR');
    } elseif (getenv('HTTP_CLIENT_IP')) {
        $realip = getenv('HTTP_CLIENT_IP');
    } else {
        $realip = getenv('REMOTE_ADDR');
    }

    $realip = explode(',', $realip);
    if($realip[0] === '::1')
    {
        return '127.0.0.1';
    }
    return $realip[0];
}

function doSign($_srcData,$_key)
{
    /*
     * 1、拼装签名原始串
     *  1.1、除sign字段之外，所有参数按照字段名的ascii码从小到大排序后使用QueryString的格式（key1=value1&key2=value2）拼接而成
     *  1.2、空值不传递，不参与签名组串
     *  1.3、签名原始串中，字段名和字段值都采用原始值，不进行url encode
     * 2、生成签名
     *  2.1、MD5签名
     *  2.2、sign = md5(签名原始串&key=商户秘钥).toUpperCase
     */

    //  step 1.1
    ksort($_srcData);

    $signStr = "";
    foreach($_srcData as $k => $v) {
        if("sign" != $k && "" != $v) {
            $signStr .= $k . "=" . $v . "&";
        }
    }

    //  step 2.2
    $signStr .= "secret=" . $_key;
    //file_put_contents('/work/logs/sign_test', 'signStr *** '.$signStr.PHP_EOL,FILE_APPEND);
    $sign = strtoupper(md5($signStr));

    return $sign;
}
/***************************************************************************************************************************************************************************
 * 加解密
 * 函数authcode($string, $operation, $key, $expiry)中的$string：字符串，明文或密文；$operation：DECODE表示解密，其它表示加密；$key：密匙；$expiry：密文有效期。
 *  $str = 'abcdef'; 
    $key = 'www.helloweba.com'; 
    echo authcode($str,'ENCODE',$key,0); //加密 
    $str = '56f4yER1DI2WTzWMqsfPpS9hwyoJnFP2MpC8SOhRrxO7BOk'; 
    echo authcode($str,'DECODE',$key,0); //解密
 ***************************************************************************************************************************************************************************/
function authcode($string, $operation = 'DECODE', $key = '', $expiry = 0) {   
    // 动态密匙长度，相同的明文会生成不同密文就是依靠动态密匙   
    $ckey_length = 4;   
       
    // 密匙   
    $key = md5($key ? $key : $GLOBALS['discuz_auth_key']);   
       
    // 密匙a会参与加解密   
    $keya = md5(substr($key, 0, 16));   
    // 密匙b会用来做数据完整性验证   
    $keyb = md5(substr($key, 16, 16));   
    // 密匙c用于变化生成的密文   
    $keyc = $ckey_length ? ($operation == 'DECODE' ? substr($string, 0, $ckey_length): 
    substr(md5(microtime()), -$ckey_length)) : '';   
    // 参与运算的密匙   
    $cryptkey = $keya.md5($keya.$keyc);   
    $key_length = strlen($cryptkey);   
    // 明文，前10位用来保存时间戳，解密时验证数据有效性，10到26位用来保存$keyb(密匙b)， 
    //解密时会通过这个密匙验证数据完整性   
    // 如果是解码的话，会从第$ckey_length位开始，因为密文前$ckey_length位保存 动态密匙，以保证解密正确   
    $string = $operation == 'DECODE' ? base64_decode(substr($string, $ckey_length)) :  
    sprintf('%010d', $expiry ? $expiry + time() : 0).substr(md5($string.$keyb), 0, 16).$string;   
    $string_length = strlen($string);   
    $result = '';   
    $box = range(0, 255);   
    $rndkey = array();   
    // 产生密匙簿   
    for($i = 0; $i <= 255; $i++) {   
        $rndkey[$i] = ord($cryptkey[$i % $key_length]);   
    }   
    // 用固定的算法，打乱密匙簿，增加随机性，好像很复杂，实际上对并不会增加密文的强度   
    for($j = $i = 0; $i < 256; $i++) {   
        $j = ($j + $box[$i] + $rndkey[$i]) % 256;   
        $tmp = $box[$i];   
        $box[$i] = $box[$j];   
        $box[$j] = $tmp;   
    }   
    // 核心加解密部分   
    for($a = $j = $i = 0; $i < $string_length; $i++) {   
        $a = ($a + 1) % 256;   
        $j = ($j + $box[$a]) % 256;   
        $tmp = $box[$a];   
        $box[$a] = $box[$j];   
        $box[$j] = $tmp;   
        // 从密匙簿得出密匙进行异或，再转成字符   
        $result .= chr(ord($string[$i]) ^ ($box[($box[$a] + $box[$j]) % 256]));   
    }   
    if($operation == 'DECODE') {  
        // 验证数据有效性，请看未加密明文的格式   
        if((substr($result, 0, 10) == 0 || substr($result, 0, 10) - time() > 0) &&  
            substr($result, 10, 16) == substr(md5(substr($result, 26).$keyb), 0, 16)) {   
            return substr($result, 26);   
        } else {   
            return '';   
        }   
    } else {   
        // 把动态密匙保存在密文里，这也是为什么同样的明文，生产不同密文后能解密的原因   
        // 因为加密后的密文可能是一些特殊字符，复制过程可能会丢失，所以用base64编码   
        return $keyc.str_replace('=', '', base64_encode($result));   
    }   
} 
/***************************************************************************************************************************************************************************
 * 随机数
 ***************************************************************************************************************************************************************************/
function nonce_str()
{
    return uniqid();
}
/***************************************************************************************************************************************************************************
 * 参数判断
 ***************************************************************************************************************************************************************************/
/**
 * 判断变量是否为空
 * @param type $_parm
 * @return boolean
 */
function isVarEmpty($_parm)
{
    $res = false;
    $empty = array('',null,false);
    if(in_array($_parm,$empty))
    {
        $res = true;
    }
    else if(is_array($_parm) && count($_parm)<1)
    {
        $res = true;
    }
    else if(is_object($_parm) && empty($_parm))
    {
        $res = true;
    }
    return $res;
}
/***************************************************************************************************************************************************************************
 * HTTP请求
 ***************************************************************************************************************************************************************************/
/**
 * 发起http GET请求，并返回数据
 * @param type $_uri
 * @return type
 */
function httpGetAndReturn($_uri)
{
    $ch = curl_init($_uri);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true) ; // 获取数据返回  
    curl_setopt($ch, CURLOPT_BINARYTRANSFER, true) ; // 在启用 CURLOPT_RETURNTRANSFER 时候将获取数据返回  
    $output = curl_exec($ch);
    curl_close($ch);

    return $output;
}
/**
 * 发起post请求，并返回数据
 * @param type $_uri
 */
function httpPostAndReturn($_uri,$_params)
{
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL,$_uri) ;  
    curl_setopt($ch, CURLOPT_POST,count($_params)) ; // 启用时会发送一个常规的POST请求，类型为：application/x-www-form-urlencoded，就像表单提交的一样。  
    curl_setopt($ch, CURLOPT_POSTFIELDS,$_params); // 在HTTP中的“POST”操作。如果要传送一个文件，需要一个@开头的文件名  

    ob_start();
    curl_exec($ch);
    $result = ob_get_contents();
    ob_end_clean();
    //close connection  
    curl_close($ch); 
    
    return $result;
}
/**
 * 发起post请求，并返回数据
 * @param type $_uri
 */
function httpPostNoReturn($_uri,$_params)
{
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL,$_uri);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, false);
    curl_setopt($ch, CURLOPT_POST,count($_params)); // 启用时会发送一个常规的POST请求，类型为：application/x-www-form-urlencoded，就像表单提交的一样。  
    curl_setopt($ch, CURLOPT_POSTFIELDS,$_params); // 在HTTP中的“POST”操作。如果要传送一个文件，需要一个@开头的文件名  
    curl_exec($ch);
    curl_close($ch);
    exit();
}
/***************************************************************************************************************************************************************************
 * 请求客户端判断
 ***************************************************************************************************************************************************************************/
/**
 * 获取当前平台
 * 通过http报头判断
 */
function isWeixin()
{
    if ( strpos(htmlspecialchars($_SERVER['HTTP_USER_AGENT']), 'MicroMessenger') !== false ) {
        return true;
    }	
    return false;
}
/**
 * 获取设备平台
 */
function getDevicePlatform()
{
    /*
     * 1、安卓，返回'AND_WAP'
     * 2、苹果，返回'IOS_WAP'
     */
    $agent = strtolower(htmlspecialchars($_SERVER['HTTP_USER_AGENT']));
    if(strpos($agent, 'iphone') || strpos($agent, 'ipad'))
    {
        return 'IOS_WAP';
    }
    else if(isWeixin())
    {
        return 'WX_WAP';
    }
    return 'AND_WAP';
}


//返回随机密码
function cai_get_pwd($mylen=8){
    //密码长度:$mylen
    if(empty($mylen)){
        $mylen = 8;
    }
    $array="2123456789";
    $authnum = '';
    for($i=0;$i<$mylen;$i++){
        $authnum .=substr($array,mt_rand(0,9),1);
    }
    return 	$authnum;
}

//解码(主要针对特殊符号和emoji表情)
function emoji_decode($str){
    if(empty($str)){
        return $str;
    }
    $str = preg_replace_callback('/@E(.{6}==)/', function ($r) {
        return base64_decode($r[1]);
    }, $str);
    return $str;
}

function createLoginToken($data)
{
    $arr = json_encode($arr);
    return base64_encode(uc_authcode($arr, 'ENCODE'));
}

function decryptToken($token)
{
    if($token){
        $val = uc_authcode(base64_decode($token),'DECODE');
        if(!empty($val)){
            $val = json_decode($val,true);
        }else{
            $val = false;
        }
        return $val;
    }else{
        return false;
    }
}

/**
 *功能：加密解密函数
 *@param string $string 要加密字符串
 *@param string $operation 动作 DECODE 解密， ENCODE 加密
 *@param string $key 加密key
 *@param int expiry
 *@return string 加密或解密的字符串
 */
function uc_authcode($string, $operation = 'DECODE', $key = '', $expiry = 0) {
    $ckey_length = 4;
    $key = md5($key ? $key : '12R,sf2$1U5)kk#UI9.wiu');
    $keya = md5(substr($key, 0, 16));
    $keyb = md5(substr($key, 16, 16));
    $keyc = $ckey_length ? ($operation == 'DECODE' ? substr($string, 0, $ckey_length): substr(md5(microtime()), -$ckey_length)) : '';

    $cryptkey = $keya.md5($keya.$keyc);
    $key_length = strlen($cryptkey);

    $string = $operation == 'DECODE' ? base64_decode(substr($string, $ckey_length)) : sprintf('%010d', $expiry ? $expiry + time() : 0).substr(md5($string.$keyb), 0, 16).$string;
    $string_length = strlen($string);

    $result = '';
    $box = range(0, 255);

    $rndkey = array();
    for($i = 0; $i <= 255; $i++) {
        $rndkey[$i] = ord($cryptkey[$i % $key_length]);
    }

    for($j = $i = 0; $i < 256; $i++) {
        $j = ($j + $box[$i] + $rndkey[$i]) % 256;
        $tmp = $box[$i];
        $box[$i] = $box[$j];
        $box[$j] = $tmp;
    }

    for($a = $j = $i = 0; $i < $string_length; $i++) {
        $a = ($a + 1) % 256;
        $j = ($j + $box[$a]) % 256;
        $tmp = $box[$a];
        $box[$a] = $box[$j];
        $box[$j] = $tmp;
        $result .= chr(ord($string[$i]) ^ ($box[($box[$a] + $box[$j]) % 256]));
    }

    if($operation == 'DECODE') {
        if((substr($result, 0, 10) == 0 || substr($result, 0, 10) - time() > 0) && substr($result, 10, 16) == substr(md5(substr($result, 26).$keyb), 0, 16)) {
            return substr($result, 26);
        } else {
            return '';
        }
    } else {
        return $keyc.str_replace('=', '', base64_encode($result));
    }
}

function curlPost($url,$data='',$is_ssl = false) {
    $ch = curl_init();
    // 配置
    curl_setopt($ch, CURLOPT_URL,$url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
    if ($is_ssl) {
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0); // 对认证证书来源的检查
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2); // 从证书中检查SSL加密算法是否存在
        curl_setopt($ch, CURLOPT_SSLVERSION, 1); // 设定SSL版本,微信时候使用1，否则报错
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
    }

    // 执行
    $re = curl_exec($ch);

    // 关闭
    curl_close($ch);
    return $re;
}

function getTime() {
	list($microsecond , $time) = explode(' ', microtime()); //' '中间是一个空格 
    return (float)sprintf('%.0f',(floatval($microsecond)+floatval($time))*1000);
}