<?php
/*
 * 统计 - 文本日志记录
 *  每天一个文件
 */
namespace Home\Service;

class LogService
{
	 /* @var int $logLevel 多个日记级别
	 */
    protected $logLevel = 0;

    /**
     * @var int LOG_LEVEL_DEBUG 调试级别
     */
    const LOG_LEVEL_DEBUG = 1;

    /**
     * @var int LOG_LEVEL_INFO 产品级别
     */
    const LOG_LEVEL_INFO = 2;

    /**
     * @var int LOG_LEVEL_ERROR 错误级别
     */
    const LOG_LEVEL_ERROR = 4;

    protected $logFolder = '/opt/log/game/';

    protected $dateFormat = 'Y-m-d H:i:s';
    protected $fileDate;

    public function init()
    {
        $conf = C("log");

        if(isset($conf['logFolder']) && !empty($conf['logFolder'])) $this->logFolder = $conf['logFolder'];
        if(isset($conf['dateFormat']) && !empty($conf['dateFormat'])) $this->dateFormat = $conf['dateFormat'];

        // 跨天时新建日记文件
        $curFileDate = date('Y-m-d', time());

        #减轻套在循环中的执行次数
        if ($this->fileDate == $curFileDate) {
            return;
        }
        $this->fileDate = $curFileDate;

        // 每月一个目录
        $folder = $this->logFolder;

        $folder = rtrim($folder,'/').'/'; //去掉最后一个/

        if (!file_exists($folder)) {
            mkdir($folder, 0777, TRUE);
        }

        // 每天一个文件
        $this->logFile = $folder . $this->fileDate;
        if (!file_exists($this->logFile)) {
            // 当没有权限时，touch会抛出(Permission denied)异常
            @touch($this->logFile);
            // touch失败时，chmod会抛出(No such file or directory)异常
            if (file_exists($this->logFile)) {
                chmod($this->logFile, 0777);
            }
        }
    }

     public function log($type, $msg, $data) {

        $this->init();

        $msgArr = array();
        $msgArr[] = date($this->dateFormat, time());
        $msgArr[] = strtoupper($type);
        $msgArr[] = str_replace(PHP_EOL, '\n', $msg);
        if ($data !== NULL) {
            $msgArr[] = is_array($data) ? json_encode($data) : $data;
        }
        $msgArr[] = get_client_ip(); //加上ip;

        $content = implode('|', $msgArr) . PHP_EOL;

        return file_put_contents($this->logFile, $content, FILE_APPEND);
    }


    /**
     * 应用产品级日记
     * @param string $msg 日记关键描述
     * @param string/array $data 场景上下文信息
     * @return NULL
     */
    public function info($msg, $data = NULL) {
        /*if (!$this->isAllowToLog(self::LOG_LEVEL_INFO)) {
            return;
        }*/

        $this->log('info', $msg, $data);
    }

    /**
     * 开发调试级日记
     * @param string $msg 日记关键描述
     * @param string/array $data 场景上下文信息
     * @return NULL
     */
    public function debug($msg, $data = NULL) {
       /* if (!$this->isAllowToLog(self::LOG_LEVEL_DEBUG)) {
            return;
        }*/

        $this->log('debug', $msg, $data);
    }

    /**
     * 系统错误级日记
     * @param string $msg 日记关键描述
     * @param string/array $data 场景上下文信息
     * @return NULL
     */
    public function error($msg, $data = NULL) {
        /*if (!$this->isAllowToLog(self::LOG_LEVEL_ERROR)) {
            return;
        }*/

        $this->log('error', $msg, $data);
    }

    /**
     * 是否允许写入日记，或运算
     * @param int $logLevel
     * @return boolean
     */
    /*protected function isAllowToLog($logLevel) {
        return (($this->logLevel & $logLevel) != 0) ? TRUE : FALSE;
    }*/
}