<?php
namespace Home\Utils;

class LogUtil{
   const LOG_DIR = '/opt/logs/';
   static $project = 'ylc.llewan.com';
   
   /*
    * 输入记录
    * @param string $project_name 项目名称/目录名称
    */
   public static function input($project_name){
   		if($project_name){
   			self::$project = $project_name;
   		}
   		
	   	$dir = self::getDir();
	   	$filename = date('Ymd').'.log';
	   	$fullPath = $dir.$filename;
	   	
	   	$logdata['request_type'] =  $_SERVER['REQUEST_METHOD'];
		$logdata['params'] = self::get_request_data();
   	    $logdata['client']['ip'] = self::get_real_ip();
   	    $logdata['client']['device'] = self::isCellPhone() ? 'mobile' : 'pc';
   	    $logdata['client']['device_type'] = self::get_device_type();
   	    $logdata['client']['app'] = self::get_app_type();
   	    $logdata['client']['refer'] = $_SERVER['HTTP_REFERER'];
       	$logdata['client']['agent'] = $_SERVER['HTTP_USER_AGENT'];
       	$input_url = self::get_current_url();
   	    $logdata['server']['ip'] = self::get_server_ip($input_url);
   	    $logdata['server']['name'] = $_SERVER['SERVER_NAME']; 
   	    
	   	$date_str = '['.date('Y-m-d H:i:s').']';

	   	$content = $date_str."input url:".$input_url.PHP_EOL;
	   
	   	foreach($logdata as $key => $val){
	   		$content .= $date_str.str_pad($key, 15)."=\t".self::transToString($val).PHP_EOL;
	   	}
   		
	   	$content .= $date_str.str_pad('input end',50,'*',STR_PAD_BOTH).PHP_EOL;
	   	
	   	return file_put_contents($fullPath,$content,FILE_APPEND);
   }
   
   /*
    * 业务输入记录
    * @param mixed  $data 记录数据
    * @param string $file_name   自定义文件名
    */
   public static function save($data='',$file_name=''){
	   	$dir = self::getDir();
	   	$filename = $file_name ? $file_name : date('Ymd').'.log';
   		$fullPath = $dir.$filename;
	   	 
	   	$date_str = '['.date('Y-m-d H:i:s').']';

	    $content = "";
	   	if(is_array($data)){
	   		foreach($data as $key => $val){
	   			if(is_numeric($key)){
	   				$content .= $date_str.self::transToString($val).PHP_EOL;
	   			}else{
	   				$content .= $date_str.str_pad($key, 15)."=\t".self::transToString($val).PHP_EOL;
	   			}
	   			
	   		}
	   	}else{
	   		$content = $date_str.$data.PHP_EOL;
	   	}
	   
	   	 
	   	return file_put_contents($fullPath,$content,FILE_APPEND);
   }
    
   /*
    * 页面输入记录
    * @param mixed  $data 记录数据
    * @param string $project_name  项目名称/目录名称
    */
   public static function output($data=''){
   		$dir = self::getDir();
	   	$filename = date('Ymd').'.log';
	   	$fullPath = $dir.$filename;
	   	
	   $date_str = '['.date('Y-m-d H:i:s').']';
	   
	   $content = $date_str.'output data:'.PHP_EOL;

	   if(isset($data['project_name'])){
	   	  unset($data['project_name']);
	   }

	   foreach((array)$data as $key => $val){

	        if(empty($val)) continue;
	   		if(is_numeric($key)){
	   			$content .= $date_str.self::transToString($val).PHP_EOL;
	   		}else{
	   			$content .= $date_str.str_pad($key, 15)."=\t".self::transToString($val).PHP_EOL;
	   		}
	   		
	   }
	   	
	   $cut_off_line = $date_str.str_pad('output end',50,'*',STR_PAD_BOTH).PHP_EOL;
	   $content .= $cut_off_line;
	   
	   return file_put_contents($fullPath,$content,FILE_APPEND);
   }
   
   /**
    * 接口记录和返回
    * @param mixed $data
    * @param string $type
    * @param int $option json或xml的option参数
    */
   static public function retMsg($data,$type='json',$option=''){
   		//返回数据
   		switch (strtoupper($type)){
   			case 'JSON' :
   				// 返回JSON数据格式到客户端 包含状态信息
   				header('Content-Type:application/json; charset=utf-8');
   				$data       =   json_encode($data,$option ? : 0);
   				break;
   			case 'JSONP':
   				// 返回JSON数据格式到客户端 包含状态信息
   				header('Content-Type:application/json; charset=utf-8');
   				$handler    =   isset($_GET['callback']) ? $_GET['callback'] : 'jsonpReturn';
   				$data       =   $handler.'('.json_encode($data,$option).');';
   				break;
   			case 'XML'  :
   				// 返回xml格式数据
   				header('Content-Type:text/xml; charset=utf-8');
   				$root = $option['root'] ? $option['root'] : 'xml';
   				$item = $option['item'] ? $option['item'] : 'item';
   				$attr = $option['attr'] ? $option['attr'] : '';
   				$id = $option['id'] ? $option['id'] : 'id';
   				$encoding = $option['encoding'] ? $option['encoding'] : 'utf-8';
   				$data = self::xml_encode($data,$root,$item,$attr,$id,$encoding);
   			case 'EVAL' :
   			default :
   				// 返回可执行的js脚本
   				header('Content-Type:text/html; charset=utf-8');
   				break;
   		}
   		
   		//记录
   		$logdata['return_data_type'] = $type;
   		$tmp = is_string($data) ? array('return_data'=>$data) : $data;
   		$logdata = array_merge($logdata,$tmp);
   		self::output($logdata);
   		
   		exit($data);
   }
   
   /**
    * 接口同一返回函数
    * @param $msg 
    * @param $code
    * @param $data
    */
   static public function jsonReturn($msg,$code=0,$data=''){
   		$ret = array('errcode'=>$code,'msg'=>$msg,'data'=>$data);
   		
   		self::retMsg($ret);
   }
   	/**
   	 * 请求接口记录
   	 * @param string $url 请求地址
   	 * @param mixed  $parmas 请求数据
   	 * @param mixed  $ret  接口返回
   	 * @param type  string  get/post
   	 * @param filename string  保存文件名
   	 */
   static public function requestLog($url,$params='',$ret='',$type='get',$filename=''){
   		$dir = self::getDir();
   		
   		/*
   		if(!$filename){
   			$path = parse_url($url);
   			$filename = $path['host'].'_'.str_replace('/', '_', $path['path']).'.log';
   		}
   		*/
   		$filename = date('Ymd').'.log';
   		$fullPath = $dir.$filename;
   		//exit($fullPath);
   		$date_str = '['.date('Y-m-d H:i:s').']';
	   
	   	$content = $date_str.str_pad('request_start',40,'*',STR_PAD_BOTH).PHP_EOL;
	   	$content .= $date_str.'请求地址:'.$url.PHP_EOL;
	   	$content .= $date_str.'请求方式:'.strtoupper($type).PHP_EOL;
	   	if(is_string($params)){
	   		$content .= $date_str.'请求数据:'.$params.PHP_EOL;
	   	}else{
	   		$content .= $date_str.'请求数据:'.PHP_EOL;
	   		
	   		foreach($params as $key => $val){
	   			$content .= $date_str.str_pad($key,15).'='."\t".$val.PHP_EOL;
	   		}
	   	}
	   	
	   	if(is_string($ret)){
	   		$content .= $date_str.'返回数据:'.$ret.PHP_EOL;
	   	}else{
	   		$content .= $date_str.'返回数据:'.PHP_EOL;
	   	
	   		foreach($ret as $k => $v){
	   			$content .= $date_str.str_pad($k,15).'='."\t".$v.PHP_EOL;
	   		}
	   	}
	   	
	   	$content .= $date_str.str_pad('request_end',40,'*',STR_PAD_BOTH).PHP_EOL;
	   	return file_put_contents($fullPath, $content,FILE_APPEND);
   }
   
   /**
    * 一般记录
    * @param mixed  $data 记录数据
    * @param string $filename   自定义文件名
    **/
   public static function debug($content='',$filename=''){
	   	$dir = self::getDir();
	   	$filename = $filename ? : 'debug.log';
	   	$fullPath = $dir.$filename;
	   	
	   	$logdata['request_type'] =  $_SERVER['REQUEST_METHOD'];
	   	$logdata['params'] = self::get_request_data();
	   	$logdata['client']['ip'] = self::get_real_ip();
	   	$logdata['client']['device'] = self::isCellPhone() ? 'mobile' : 'pc';
	   	$logdata['client']['device_type'] = self::get_device_type();
	   	$logdata['client']['app'] = self::get_app_type();
	   	$logdata['client']['refer'] = $_SERVER['HTTP_REFERER'];
	   	$logdata['client']['agent'] = $_SERVER['HTTP_USER_AGENT'];
	   	
	   	$logdata['server']['ip'] = self::get_server_ip($logdata['uri']);
	   	$logdata['server']['name'] = $_SERVER['SERVER_NAME'];
	   	$input_url = self::get_current_url();
	   	$date_str = '['.date('Y-m-d H:i:s').']';
	   	
	   	$content = $date_str.str_pad('debug start',50,'*',STR_PAD_BOTH).PHP_EOL;
	   	$content .= $date_str."FILE\t:\t".__FILE__.PHP_EOL;
	   	$content .= $date_str."URL\t:\t".$input_url.PHP_EOL;
	   	foreach($logdata as $key => $val){
	   		$content .= $date_str.str_pad($key, 15)."=\t".self::transToString($val).PHP_EOL;
	   	}
	   	 
	   	$input_url = self::get_current_url();
	   	$date_str = '['.date('Y-m-d H:i:s').']';
	  
	   	return file_put_contents($fullPath,$content,FILE_APPEND);
   }
   
   /**
    * 得到目录文件
    */
   static private function getDir(){
	   	$project_name = self::$project;
	   	$dir = self::LOG_DIR.rtrim($project_name,'/\\').'/';
	   	
	   	if(!is_dir($dir)){
	   		mkdir($dir,0777,true);
	   		@chmod($dir,0777);
	   	}
   		
	   	return $dir;
   }
   
    /**
    * 获取请求数据
    */
   static public  function get_request_data(){
   		$input = file_get_contents('php://input');
   		
   		/*$data = json_decode($input,true);
   		
   		if(empty($data)){
   			parse_str($input,$data);
   		}*/
   		if(self::is_json($input)){
   			$data = json_decode($input,true);
   		}else{
   			parse_str($input,$data);
   		}
   		
   		return array_merge(I('get.',array()),I('post.',array()),$data);
   }
   
   /**
    * XML编码
    * @param mixed $data 数据
    * @param string $root 根节点名
    * @param string $item 数字索引的子节点名
    * @param string $attr 根节点属性
    * @param string $id   数字索引子节点key转换的属性名
    * @param string $encoding 数据编码
    * @return string
    */
   static public function xml_encode($data, $root='xml', $item='item', $attr='', $id='id', $encoding='utf-8') {
   	if(is_array($attr)){
   		$_attr = array();
   		foreach ($attr as $key => $value) {
   			$_attr[] = "{$key}=\"{$value}\"";
   		}
   		$attr = implode(' ', $_attr);
   	}
   	$attr   = trim($attr);
   	$attr   = empty($attr) ? '' : " {$attr}";
   	$xml    = "<?xml version=\"1.0\" encoding=\"{$encoding}\"?>";
   	$xml   .= "<{$root}{$attr}>";
   	$xml   .= self::data_to_xml($data, $item, $id);
   	$xml   .= "</{$root}>";
   	return $xml;
   }
   
   /**
    * 数据XML编码
    * @param mixed  $data 数据
    * @param string $item 数字索引时的节点名称
    * @param string $id   数字索引key转换为的属性名
    * @return string
    */
   static public function data_to_xml($data, $item='item', $id='id') {
	   	$xml = $attr = '';
	   	foreach ($data as $key => $val) {
	   		if(is_numeric($key)){
	   			$id && $attr = " {$id}=\"{$key}\"";
	   			$key  = $item;
	   		}
	   		$xml    .=  "<{$key}{$attr}>";
	   		$xml    .=  (is_array($val) || is_object($val)) ? data_to_xml($val, $item, $id) : $val;
	   		$xml    .=  "</{$key}>";
	   	}
	   	return $xml;
   }
   
   static public function transToString($data){
   		if(is_string($data)){
   			return $data;
   		}else{
   			return json_encode($data,JSON_UNESCAPED_UNICODE);
   		}
   }
   
   /*
    * 获取当前页面完整URL地址
    */
   static public function get_current_url() {
	   	$sys_protocal = isset($_SERVER['SERVER_PORT']) && $_SERVER['SERVER_PORT'] == '443' ? 'https://' : 'http://';
	   	$php_self = $_SERVER['PHP_SELF'] ? $_SERVER['PHP_SELF'] : $_SERVER['SCRIPT_NAME'];
	   	$path_info = isset($_SERVER['PATH_INFO']) ? $_SERVER['PATH_INFO'] : '';
	   	$relate_url = isset($_SERVER['REQUEST_URI']) ? $_SERVER['REQUEST_URI'] : $php_self.(isset($_SERVER['QUERY_STRING']) ? '?'.$_SERVER['QUERY_STRING'] : $path_info);
	   	//return $sys_protocal.(isset($_SERVER['HTTP_HOST']) ? $_SERVER['HTTP_HOST'] : '').$relate_url;
	   	$domain = $_SERVER['SERVER_NAME'] ? $_SERVER['SERVER_NAME'] : isset($_SERVER['HTTP_HOST']) ? $_SERVER['HTTP_HOST'] : '';
	   	return $sys_protocal.$domain.$relate_url;
   }
   
   
   /*
    * 判断是否移动端打开
    */
   static public function isCellPhone(){
   	// 如果有HTTP_X_WAP_PROFILE则一定是移动设备
   	if (isset ($_SERVER['HTTP_X_WAP_PROFILE']))
   	{
   		return true;
   	}
   	// 如果via信息含有wap则一定是移动设备,部分服务商会屏蔽该信息
   	if (isset ($_SERVER['HTTP_VIA']))
   	{
   		// 找不到为flase,否则为true
   		return stristr($_SERVER['HTTP_VIA'], "wap") ? true : false;
   	}
   	// 脑残法，判断手机发送的客户端标志,兼容性有待提高
   	if (isset ($_SERVER['HTTP_USER_AGENT']))
   	{
   		$clientkeywords = array ('nokia',
   				'sony',
   				'ericsson',
   				'mot',
   				'samsung',
   				'htc',
   				'sgh',
   				'lg',
   				'sharp',
   				'sie-',
   				'philips',
   				'panasonic',
   				'alcatel',
   				'lenovo',
   				'iphone',
   				'ipod',
   				'blackberry',
   				'meizu',
   				'android',
   				'netfront',
   				'symbian',
   				'ucweb',
   				'windowsce',
   				'palm',
   				'operamini',
   				'operamobi',
   				'openwave',
   				'nexusone',
   				'cldc',
   				'midp',
   				'wap',
   				'mobile'
   		);
   		// 从HTTP_USER_AGENT中查找手机浏览器的关键字
   		if (preg_match("/(" . implode('|', $clientkeywords) . ")/i", strtolower($_SERVER['HTTP_USER_AGENT'])))
   		{
   			return true;
   		}
   	}
   	// 协议法，因为有可能不准确，放到最后判断
   	if (isset ($_SERVER['HTTP_ACCEPT']))
   	{
   		// 如果只支持wml并且不支持html那一定是移动设备
   		// 如果支持wml和html但是wml在html之前则是移动设备
   		if ((strpos($_SERVER['HTTP_ACCEPT'], 'vnd.wap.wml') !== false) && (strpos($_SERVER['HTTP_ACCEPT'], 'text/html') === false || (strpos($_SERVER['HTTP_ACCEPT'], 'vnd.wap.wml') < strpos($_SERVER['HTTP_ACCEPT'], 'text/html'))))
   		{
   			return true;
   		}
   	}
   	return false;
   }
   
   /*
    * 获取设备类型
    */
   static public function get_device_type()
   {
   	//全部变成小写字母
   	$agent= strtolower($_SERVER['HTTP_USER_AGENT']);
   	$type='other';
   	//分别进行判断
   	if(strpos($agent,'iphone') || strpos($agent,'ipad'))
   	{
   		$type='ios';
   	}
   
   	if(strpos($agent,'android'))
   	{
   		$type='android';
   	}
   	return $type;
   }
   
   /*
    * 获取打开的APP
    */
   static public function get_app_type()
   {
   	//全部变成小写字母
   	$agent= strtolower($_SERVER['HTTP_USER_AGENT']);
   	$app ='other';
   	if(strpos($agent,"micromessenger")) //微信
   	{
   		$app = '微信';
   	}
   	else if(strpos($agent,"weibo")) //微博
   	{
   		$app = '微博';
   	}
   	else if(strpos($agent,"aliapp")) //来往
   	{
   		$app = '来往';
   	}
   
   	return $app;
   }
   
   /*
    * 获取服务器IP
    */
   static public  function get_server_ip($domian){
	   	$pathinfo = parse_url($domian);
	   	$server = isset($pathinfo['host']) ? $pathinfo['host'] : $pathinfo['path'];
	   	$server_ip = gethostbyname($server);
	   	return $server_ip ? : $_SERVER['SERVER_ADDR'];
   }
   
   static public function get_real_ip(){
	   	if (isset($_SERVER['HTTP_CDN_SRC_IP']) && $_SERVER['HTTP_CDN_SRC_IP'] != 'unknown') {
	   		$realip = $_SERVER["HTTP_CDN_SRC_IP"];
	   	} elseif($_SERVER['HTTP_X_REAL_IP']){
	   		$realip = $_SERVER['HTTP_X_REAL_IP'];
	   	} elseif ($_SERVER['HTTP_X_FORWARDED_FOR']) {
	   		$realip = $_SERVER['HTTP_X_FORWARDED_FOR'];
	   	} elseif ($_SERVER['HTTP_CLIENT_IP']) {
	   		$realip = $_SERVER['HTTP_CLIENT_IP'];
	   	}elseif ($_SERVER['REMOTE_ADDR']){
	   		$realip = $_SERVER['REMOTE_ADDR'];
	   	}elseif (getenv('HTTP_X_FORWARDED_FOR')) {
	   		$realip = getenv('HTTP_X_FORWARDED_FOR');
	   	} elseif (getenv('HTTP_CLIENT_IP')) {
	   		$realip = getenv('HTTP_CLIENT_IP');
	   	} else {
	   		$realip = getenv('REMOTE_ADDR');
	   	}
	   	
	   	$realip = explode(',', $realip);
	   	if($realip[0] === '::1')
	   	{
	   		return '127.0.0.1';
	   	}
	   	
	   	return $realip[0];
	}
	
    //是否json数据
    static public function is_json($str)
    {
        if(!is_string($str)) return false;

        if(preg_match('/[^,:{}\\[\\]0-9.\-+Eaeflnr-u \n\r\t]/',$str)){

            return !is_null(json_decode($str));
        }
        return false;
    }
}